Package: RnBeads
Title: RnBeads
Description: RnBeads facilitates comprehensive analysis of various types of DNA
    methylation data at the genome scale.
Authors@R: c(
    person("Yassen", "Assenov", email = "assenov@gmail.com",
    role = c("aut")),
    person("Christoph", "Bock", email = "cbock@cemm.at",
    role = c("aut")),
    person("Pavlo", "Lutsik", email = "p.lutsik@dkfz.de",
    role = c("aut")),
    person("Michael", "Scherer", email = "mscherer@mpi-inf.mpg.de",
    role = c("aut")),
    person("Fabian", "Mueller", email = "team@rnbeads.org",
    role = c("aut", "cre")))
Date: 2026-07-16
Version: 2.31.2
Suggests: Category, GOstats, Gviz,
        IlluminaHumanMethylation450kmanifest, RPMM, RnBeads.hg19,
        RnBeads.mm9, RnBeads.hg38, XML, annotate, biomaRt, foreach,
        doParallel, ggbio, isva, mclust, mgcv, minfi, nlme,
        org.Hs.eg.db, org.Mm.eg.db, org.Rn.eg.db, quadprog,
        rtracklayer, qvalue, sva, wateRmelon, wordcloud, qvalue,
        argparse, glmnet, IlluminaHumanMethylation450kanno.ilmn12.hg19,
        scales, missMethyl, impute, shiny, shinyjs, plotrix, hexbin,
        RUnit, MethylSeekR, sesame, dplyr
Depends: R (>= 4.1.0), BiocGenerics, S4Vectors (>= 0.9.25),
        GenomicRanges, MASS, cluster, ff, fields, ggplot2 (>= 0.9.2),
        gplots, grid, gridExtra, limma, matrixStats, methods,
        illuminaio, methylumi, plyr, reshape2
Imports: IRanges, ComplexHeatmap
License: GPL-3
biocViews: DNAMethylation, MethylationArray, MethylSeq, Epigenetics,
        QualityControl, Preprocessing, BatchEffect,
        DifferentialMethylation, Sequencing, CpGIsland, ImmunoOncology,
        TwoChannel, DataImport
Collate: 'CNV.R' 'Report-class.R' 'Report-methods.R'
        'ReportPlot-class.R' 'ReportPlot-methods.R'
        'RnBDiffMeth-class.R' 'bigFf.R' 'RnBSet-class.R'
        'RnBeadSet-class.R' 'RnBeadRawSet-class.R' 'RnBeads-package.R'
        'RnBiseqSet-class.R' 'agePrediction.R' 'annotations.R'
        'batch.R' 'batch.quality.R' 'bmiq.R' 'cellTypeAdjustment.R'
        'clusterArchitecture.R' 'clusterArchitectureLSF.R'
        'clusterArchitectureSGE.R' 'clusterArchitectureSLURM.R'
        'clustering.R' 'combining.R' 'computeCluster.R'
        'controlPlots.R' 'controlPlotsBiSeq.R' 'dataExport.R'
        'dataImport.R' 'differentialMethylation.R'
        'differentialVariability.R' 'enmix.R' 'enrichment.R'
        'exportGEO.R' 'filtering.R' 'filteringSummary.R' 'genomewide.R'
        'greedycut.R' 'immune.R' 'loading.R' 'logger.R' 'lolaUtils.R'
        'main.R' 'normalization.R' 'normalizationMethods.R' 'options.R'
        'parallelProcessing.R' 'plottingUtils.R' 'profiles.R'
        'qualityControl.R' 'readGEO.R' 'regionDescription.R'
        'regionProfiles.R' 'segmentation.R' 'sesame.R' 'sex.R'
        'subSegments.R' 'sva.R' 'utilities.R' 'wbcInference.R'
        'xenomix.R'
RoxygenNote: 7.3.3
git_url: https://git.bioconductor.org/packages/RnBeads
git_branch: devel
git_last_commit: 1c6f9f8
git_last_commit_date: 2026-07-23
Repository: Bioconductor 3.24
Date/Publication: 2026-07-23
NeedsCompilation: no
Packaged: 2026-07-23 23:48:45 UTC; biocbuild
Author: Yassen Assenov [aut],
  Christoph Bock [aut],
  Pavlo Lutsik [aut],
  Michael Scherer [aut],
  Fabian Mueller [aut, cre]
Maintainer: Fabian Mueller <team@rnbeads.org>
