# Generated by roxygen2: do not edit by hand

export(BMIQ)
export(RnBeadRawSet)
export(RnBeadSet)
export(RnBiseqSet)
export(addRegionSubsegments)
export(apply.iEVORA)
export(combineTestPvalsMeth)
export(computeDiffTab.default.region)
export(computeDiffTab.default.site)
export(computeDiffTab.extended.site)
export(create.densityScatter)
export(create.scatter.dens.points)
export(createReport)
export(createReportGgPlot)
export(createReportPlot)
export(data.frame2GRanges)
export(deviation.plot.beta)
export(diffVar)
export(downloadLolaDbs)
export(estimateProportionsCP)
export(exportDMRs2regionFile)
export(get.adjustment.variables)
export(get.comparison.info)
export(get.covariates.ct)
export(get.covariates.sva)
export(get.cpg.stats)
export(get.files)
export(getCellTypesFromLolaDb)
export(getNamesFromLolaDb)
export(getTargetFromLolaDb)
export(greedycut.filter.matrix)
export(greedycut.get.statistics)
export(greedycut.get.submatrix)
export(has.covariates.ct)
export(has.covariates.sva)
export(limmaP)
export(load.region.subsegment.annotation)
export(load.rnb.diffmeth)
export(load.rnb.set)
export(loadLolaDbs)
export(logger.argument)
export(logger.close)
export(logger.completed)
export(logger.error)
export(logger.getfiles)
export(logger.info)
export(logger.isinitialized)
export(logger.machine.name)
export(logger.start)
export(logger.status)
export(logger.validate.file)
export(logger.warning)
export(lolaBarPlot)
export(lolaBoxPlotPerTarget)
export(lolaVolcanoPlot)
export(off)
export(parallel.getNumWorkers)
export(parallel.isEnabled)
export(parallel.setup)
export(parallel.teardown)
export(performGOEnrichment.diffVar)
export(performGOenrichment.diffMeth.entrez)
export(performGoEnrichment.diffMeth)
export(performLolaEnrichment.diffMeth)
export(performLolaEnrichment.diffVar)
export(prepareSOFTfileForGEO)
export(read.GS.report)
export(read.bed.files)
export(read.data.dir)
export(read.idat.files)
export(read.sample.annotation)
export(read.single.bed)
export(refFreeEWASP)
export(rnb.RnBSet.to.GRangesList)
export(rnb.RnBSet.to.bed)
export(rnb.RnBSet.to.bedGraph)
export(rnb.add.figure)
export(rnb.add.list)
export(rnb.add.paragraph)
export(rnb.add.reference)
export(rnb.add.section)
export(rnb.add.table)
export(rnb.add.tables)
export(rnb.annotation.size)
export(rnb.annotation2data.frame)
export(rnb.bed.from.segmentation)
export(rnb.beta2mval)
export(rnb.boxplot.from.segmentation)
export(rnb.build.index)
export(rnb.color.legends)
export(rnb.combine.arrays)
export(rnb.combine.seq)
export(rnb.execute.age.prediction)
export(rnb.execute.batch.qc)
export(rnb.execute.batcheffects)
export(rnb.execute.clustering)
export(rnb.execute.clustering.all)
export(rnb.execute.computeDiffMeth)
export(rnb.execute.context.removal)
export(rnb.execute.cross.reactive.removal)
export(rnb.execute.ct.estimation)
export(rnb.execute.diffVar)
export(rnb.execute.dreduction)
export(rnb.execute.export.csv)
export(rnb.execute.filter.summary)
export(rnb.execute.genomewide)
export(rnb.execute.greedycut)
export(rnb.execute.high.coverage.removal)
export(rnb.execute.high.dpval.masking)
export(rnb.execute.import)
export(rnb.execute.imputation)
export(rnb.execute.low.coverage.masking)
export(rnb.execute.lump)
export(rnb.execute.na.removal)
export(rnb.execute.normalization)
export(rnb.execute.pOOBAH)
export(rnb.execute.quality)
export(rnb.execute.segmentation)
export(rnb.execute.sex.prediction)
export(rnb.execute.sex.removal)
export(rnb.execute.snp.removal)
export(rnb.execute.sva)
export(rnb.execute.tnt)
export(rnb.execute.training)
export(rnb.execute.variability.removal)
export(rnb.export.all.annotation)
export(rnb.export.annotation)
export(rnb.export.to.ewasher)
export(rnb.export.to.trackhub)
export(rnb.find.relative.site.coord)
export(rnb.get.annotation)
export(rnb.get.assemblies)
export(rnb.get.chromosomes)
export(rnb.get.directory)
export(rnb.get.mapping)
export(rnb.get.reference)
export(rnb.get.reliability.matrix)
export(rnb.getOption)
export(rnb.infinium.control.targets)
export(rnb.initialize.reports)
export(rnb.is.option)
export(rnb.load.annotation)
export(rnb.load.annotation.from.db)
export(rnb.load.sitelist)
export(rnb.message.plot)
export(rnb.mval2beta)
export(rnb.options)
export(rnb.options2xml)
export(rnb.performance.profile)
export(rnb.plot.beta.comparison)
export(rnb.plot.betadistribution.probeCategories)
export(rnb.plot.betadistribution.sampleGroups)
export(rnb.plot.biseq.coverage)
export(rnb.plot.biseq.coverage.hist)
export(rnb.plot.biseq.coverage.violin)
export(rnb.plot.control.barplot)
export(rnb.plot.control.boxplot)
export(rnb.plot.coverage.thresholds)
export(rnb.plot.ct.heatmap)
export(rnb.plot.dreduction)
export(rnb.plot.locus.profile)
export(rnb.plot.marker.fstat)
export(rnb.plot.negative.boxplot)
export(rnb.plot.num.sites.covg)
export(rnb.plot.nv.heatmap)
export(rnb.plot.pheno.categories)
export(rnb.plot.region.profile.density)
export(rnb.plot.region.profiles)
export(rnb.plot.region.site.density)
export(rnb.plot.sentrix.distribution)
export(rnb.plot.sentrix.distributions)
export(rnb.plot.snp.barplot)
export(rnb.plot.snp.boxplot)
export(rnb.plot.snp.heatmap)
export(rnb.read.geo)
export(rnb.region.types)
export(rnb.region.types.for.analysis)
export(rnb.remove.annotation)
export(rnb.run.analysis)
export(rnb.run.differential)
export(rnb.run.dj)
export(rnb.run.example)
export(rnb.run.exploratory)
export(rnb.run.import)
export(rnb.run.inference)
export(rnb.run.preprocessing)
export(rnb.run.qc)
export(rnb.run.tnt)
export(rnb.run.xml)
export(rnb.run_xeno)
export(rnb.sample.groups)
export(rnb.sample.replicates)
export(rnb.sample.summary.table)
export(rnb.save.annotation)
export(rnb.set.annotation)
export(rnb.set.annotation.and.cpg.stats)
export(rnb.show.report)
export(rnb.step.betadistribution)
export(rnb.step.cnv)
export(rnb.write.table)
export(rnb.xml2options)
export(rowOneSampleTP)
export(rowPairedTP)
export(rowWelchP)
export(run.cross.validation)
export(save.rnb.diffmeth)
export(save.rnb.set)
export(set.covariates.ct)
export(set.covariates.sva)
exportClasses(ClusterArchitecture)
exportClasses(Report)
exportClasses(ReportGgPlot)
exportClasses(ReportPlot)
exportClasses(RnBClusterRun)
exportClasses(RnBDiffMeth)
exportClasses(RnBSet)
exportClasses(RnBeadClustering)
exportClasses(RnBeadRawSet)
exportClasses(RnBeadSet)
exportClasses(RnBiseqSet)
exportMethods(M)
exportMethods(U)
exportMethods(addDiffMethTable)
exportMethods(addPheno)
exportMethods(annotation)
exportMethods(assembly)
exportMethods(combine.rnb.sets)
exportMethods(covg)
exportMethods(destroy)
exportMethods(dpval)
exportMethods(geno)
exportMethods(get.comparison.grouplabels)
exportMethods(get.comparison.groupsizes)
exportMethods(get.comparisons)
exportMethods(get.covg.thres)
exportMethods(get.region.types)
exportMethods(get.site.test.method)
exportMethods(get.table)
exportMethods(get.variability.method)
exportMethods(getExecutable)
exportMethods(getModuleNumCores)
exportMethods(getSubCmdStr)
exportMethods(getSubCmdTokens)
exportMethods(hasCovg)
exportMethods(includes.sites)
exportMethods(initialize)
exportMethods(is.valid)
exportMethods(isImputed)
exportMethods(join.diffMeth)
exportMethods(mergeSamples)
exportMethods(meth)
exportMethods(mval)
exportMethods(nsites)
exportMethods(off)
exportMethods(pheno)
exportMethods(qc)
exportMethods(regionMapping)
exportMethods(regions)
exportMethods(reload)
exportMethods(remove.regions)
exportMethods(remove.samples)
exportMethods(remove.sites)
exportMethods(run)
exportMethods(samples)
exportMethods(save.tables)
exportMethods(setExecutable)
exportMethods(setModuleNumCores)
exportMethods(setModuleResourceRequirements)
exportMethods(sites)
exportMethods(summarize.regions)
exportMethods(summarized.regions)
exportMethods(updateMethylationSites)
exportMethods(updateRegionSummaries)
import(GenomicRanges)
import(IRanges)
import(MASS)
import(S4Vectors)
import(cluster)
import(ff)
import(fields)
import(ggplot2)
import(limma)
import(methods)
import(methylumi)
importFrom(BiocGenerics,"annotation<-")
importFrom(BiocGenerics,annotation)
importFrom(gplots,colorpanel)
importFrom(gplots,heatmap.2)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,densCols)
importFrom(grDevices,dev.control)
importFrom(grDevices,dev.off)
importFrom(grDevices,dev2bitmap)
importFrom(grDevices,pdf)
importFrom(grDevices,png)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(grDevices,xy.coords)
importFrom(graphics,abline)
importFrom(graphics,close.screen)
importFrom(graphics,layout)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,plot.new)
importFrom(graphics,polygon)
importFrom(graphics,screen)
importFrom(graphics,split.screen)
importFrom(gridExtra,arrangeGrob)
importFrom(illuminaio,readIDAT)
importFrom(matrixStats,colMaxs)
importFrom(matrixStats,colMedians)
importFrom(matrixStats,colMins)
importFrom(matrixStats,colQuantiles)
importFrom(matrixStats,colSds)
importFrom(matrixStats,colVars)
importFrom(matrixStats,rowMaxs)
importFrom(matrixStats,rowMedians)
importFrom(matrixStats,rowMins)
importFrom(matrixStats,rowQuantiles)
importFrom(matrixStats,rowSds)
importFrom(matrixStats,rowVars)
importFrom(plyr,rbind.fill)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,bartlett.test)
importFrom(stats,coef)
importFrom(stats,cutree)
importFrom(stats,dbeta)
importFrom(stats,density)
importFrom(stats,dexp)
importFrom(stats,dist)
importFrom(stats,dnorm)
importFrom(stats,ecdf)
importFrom(stats,fisher.test)
importFrom(stats,hclust)
importFrom(stats,knots)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,model.matrix)
importFrom(stats,optim)
importFrom(stats,p.adjust)
importFrom(stats,pbeta)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,qbeta)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,rt)
importFrom(stats,t.test)
importFrom(stats,vcov)
importFrom(stats,wilcox.test)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,download.file)
importFrom(utils,installed.packages)
importFrom(utils,memory.size)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,untar)
importFrom(utils,unzip)
importFrom(utils,write.table)
importFrom(utils,zip)
