# RnBeadsDJ - RnBeads Data Juggler
* __Package name:__ RnBeadsDJ
* __Title:__ RnBeads Data Juggler
* __Description:__ A Shiny app for configuring and running RnBeads analyses 
* __Author:__ Fabian Mueller
* __Maintainer:__ Fabian Mueller <fmueller@mpi-inf.mpg.de>
* __Date:__ 2017-10-04
