% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeadRawSet-class.R
\docType{methods}
\name{M,RnBeadRawSet-method}
\alias{M,RnBeadRawSet-method}
\alias{M}
\title{M-methods}
\usage{
\S4method{M}{RnBeadRawSet}(object, row.names = FALSE)
}
\arguments{
\item{object}{Dataset of interest.}

\item{row.names}{Flag indicating whether the resulting matrix will be assigned row names}
}
\value{
\code{matrix} of the methylated probe intensities
}
\description{
Extract raw methylated probe intensity from an object of \code{RnBeadRawSet} class.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
M.intensity<-M(rnb.set.example)
head(M.intensity)
} 

}
