% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/logger.R
\name{logger.isinitialized}
\alias{logger.isinitialized}
\title{logger.isinitialized}
\usage{
logger.isinitialized()
}
\value{
\code{TRUE} if the logger was initialized and is in use; \code{FALSE} otherwise.
}
\description{
Checks if the logger is initialized.
}
\examples{
\donttest{
if (!logger.isinitialized())
  logger.start(fname = NA)
}
}
\seealso{
\code{\link{logger.start}} for initializing a logger or starting a section
}
\author{
Yassen Assenov
}
