% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{updateRegionSummaries,RnBSet-method}
\alias{updateRegionSummaries,RnBSet-method}
\alias{updateRegionSummaries}
\title{updateRegionSummaries}
\usage{
\S4method{updateRegionSummaries}{RnBSet}(object)
}
\arguments{
\item{object}{Dataset of interest.}
}
\value{
Sample annotation information available for the dataset in the form of a \code{data.frame}.
}
\description{
Updates the region information present in an RnBSet by invoking summarize.regions on all region types
present in the object
}
