% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{pheno,RnBSet-method}
\alias{pheno,RnBSet-method}
\alias{pheno}
\title{pheno-methods}
\usage{
\S4method{pheno}{RnBSet}(object)
}
\arguments{
\item{object}{Dataset of interest.}
}
\value{
Sample annotation information available for the dataset in the form of a \code{data.frame}.
}
\description{
Extracts sample phenotype and/or processing information.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
pheno(rnb.set.example)
}
}
