% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBeadRawSet-class.R
\docType{methods}
\name{U,RnBeadRawSet-method}
\alias{U,RnBeadRawSet-method}
\alias{U}
\title{U-methods}
\usage{
\S4method{U}{RnBeadRawSet}(object, row.names = FALSE)
}
\arguments{
\item{object}{Dataset of interest.}

\item{row.names}{Flag indicating whether the resulting matrix will be assigned row names}
}
\value{
\code{matrix} of the unmethylated probe intensities
}
\description{
Extract raw unmethylated probe intensity from an object of \code{RnBeadRawSet} class.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
U.intensity<-U(rnb.set.example)
head(U.intensity)
} 
}
