% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{sites,RnBSet-method}
\alias{sites,RnBSet-method}
\alias{sites}
\title{sites-methods}
\usage{
\S4method{sites}{RnBSet}(object)
}
\arguments{
\item{object}{Dataset of interest.}
}
\value{
A matrix of type \code{integer} describing the sites, information for which is
present in the \code{object}
}
\description{
Methylation sites object information for which is present in the \code{RnBSet} object.
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
sites(rnb.set.example)
}
}
