% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilities.R
\name{rnb.show.report}
\alias{rnb.show.report}
\title{rnb.show.report}
\usage{
rnb.show.report(report)
}
\arguments{
\item{report}{\code{\linkS4class{Report}} object to open.}
}
\value{
None (invisible \code{NULL}).
}
\description{
Opens the given HMTL report file in the browser.
}
\author{
Pavlo Lutsik
}
