% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBDiffMeth-class.R
\docType{methods}
\name{get.region.types,RnBDiffMeth-method}
\alias{get.region.types,RnBDiffMeth-method}
\alias{get.region.types}
\title{get.region.types-methods}
\usage{
\S4method{get.region.types}{RnBDiffMeth}(object)
}
\arguments{
\item{object}{\code{\linkS4class{RnBDiffMeth}} object}
}
\value{
character vector containing region types
}
\description{
Gets all region types represented in the object as character vector
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
logger.start(fname=NA)
dm <- rnb.execute.computeDiffMeth(rnb.set.example,pheno.cols=c("Sample_Group","Treatment"))
get.region.types(dm)
}
}
\author{
Fabian Mueller
}
