% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/computeCluster.R
\docType{methods}
\name{getModuleNumCores,RnBClusterRun-method}
\alias{getModuleNumCores,RnBClusterRun-method}
\alias{getModuleNumCores}
\title{getModuleNumCores-methods}
\usage{
\S4method{getModuleNumCores}{RnBClusterRun}(object)
}
\arguments{
\item{object}{\code{\linkS4class{RnBClusterRun}} object}
}
\value{
A named vector containing the number of cores for each module
}
\description{
Retrieves the number of cores used by each module
}
\author{
Fabian Mueller
}
