% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RnBSet-class.R
\docType{methods}
\name{assembly,RnBSet-method}
\alias{assembly,RnBSet-method}
\alias{assembly}
\title{assembly-methods}
\usage{
\S4method{assembly}{RnBSet}(object)
}
\arguments{
\item{object}{Dataset of interest.}
}
\value{
Sample annotation information available for the dataset in the form of a \code{data.frame}.
}
\description{
Extracts information about assembly
}
\examples{
\donttest{
library(RnBeads.hg19)
data(small.example.object)
assembly(rnb.set.example) # "hg19"
}
}
