% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.set.annotation.and.cpg.stats}
\alias{rnb.set.annotation.and.cpg.stats}
\title{rnb.set.annotation.and.cpg.stats}
\usage{
rnb.set.annotation.and.cpg.stats(
  type,
  regions,
  description = NULL,
  assembly = "hg19"
)
}
\arguments{
\item{type, description, assembly}{Parameters handled exactly as in \code{\link{rnb.set.annotation}}}

\item{regions}{a data.frame handled similarly as by \code{\link{rnb.set.annotation}} with the exception that
the genomic location columns should be specified using upper case first letters}
}
\value{
Invisibly, \code{TRUE} if an existing annotation was replaced and \code{FALSE} otherwise.
}
\description{
wrapper for \code{\link{rnb.set.annotation}} to accept the region format as output by \code{annotation(rnb.set)}.
Additionally, CpG statistics are added to the annotation.
}
\seealso{
\code{\link{rnb.set.annotation}}
}
\author{
Fabian Mueller
}
