% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotations.R
\name{rnb.get.assemblies}
\alias{rnb.get.assemblies}
\title{rnb.get.assemblies}
\usage{
rnb.get.assemblies()
}
\value{
All supported genome assemblies in the form of a \code{character} vector. These are \code{"hg19"},
        \code{"mm10"}, \code{"mm9"} and \code{"rn5"}.
}
\description{
Gets the supported genome assemblies.
}
\examples{
\donttest{
"hg19" \%in\% rnb.get.assemblies()
}
}
\author{
Yassen Assenov
}
